% *************************************************************************
% ******************** Hyperplane Data Set Generator **********************
%          A synthetic regression problem using a drifting hyperplane
% *************************************************************************
% The Toolbox of Regression Data Sets for Concept Drifts
% Version 1.0 Nov/10/2014
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com or symonesoares@isr.uc.pt
%
% Homepage: http://sites.google.com/site/symonesoares/
%
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An On-line Weighted Ensemble 
%        of Regressor Models to Handle Concept Drifts. Engineering  
%        Applications of Artificial Intelligence, v. 37, p. 392-406, 2015. 
%     url: http://www.sciencedirect.com/science/article/pii/S09521976140024
%
% Reference: Kolter, J. Z., Maloof, M. A., August 7-11 2005. Using additive
%            Expert Ensembles to Cope With Concept Drift. In: Proc. of the 
%            22nd Int. Conf. on Machine Learning. ACM, New York, NY, USA, 
%            pp. 449–456.
%
% *************************************************************************
% *************************************************************************

clear; clc;
N = 10000;    % Define the number of samples
X = rand(N, 10);
n = floor(N / 4);  

% First Concept: y = (x1 + x2 + x3)/3
init = 1; fim  = n;

Y = (X(init:fim,1) +  X(init:fim,2) +  X(init:fim,3)) / 3;

% Second Concept: y = (x2 + x3 + x4)/3
init = fim + 1; fim = n * 2;
Y = [Y; (X(init:fim,2) +  X(init:fim,3) +  X(init:fim,4)) / 3];

% Thrid Concept: y = (x4 + x5 + x6)/3
init = fim + 1; fim = n * 3;
Y = [Y; (X(init:fim,4) +  X(init:fim,5) +  X(init:fim,6)) / 3];

% Fourth Concept: y = (x7 + x8 + x9)/3
init = fim + 1; fim = n * 4;
Y = [Y; (X(init:fim,7) +  X(init:fim,8) +  X(init:fim,9)) / 3];

% Introducing a +-0.1 random variate to y
noise = -0.1 + (0.1 + 0.1).*rand(N, 1);
Y = Y + noise;

% Clipping y if it was not in [0, 1]
ix0 = find(Y < 0);
Y(ix0) = 0;
ix1 = find(Y > 1);
Y(ix1) = 1;

input  = X;
output = Y;
save dSetHyperplane.mat input output
